        page
******************************************************
** Copyright 2003 CMI Productions.                  **
** Lots of rights reserved.                         **
** Free for personal use as long as this copyright  **
** message is maintained in the source and the LCD  **
** copyright message is also preserved.  Commercial **
** use prohibited without CMIs written concent.     **
******************************************************
; -----------------------------------------------------------------------------
; File:         a2d.asm
; Function:     These are the routines for runnning the A/D converter
; Routines:     a2dini: Initialize the A/D
;               a2dc1w: Convert 1-wire voltage
;               a2dcbt: Convert battery voltage
; -----------------------------------------------------------------------------

        page
; -----------------------------------------------------------------------------
; Routine:      a2dini
; Called By:    rstrtn
; Calls:        None
; Gazintas:     None
; Gazoutas:     None
; Function:     This routine initializes the A/D.  All that we have to do is
;               simply set the reference clock for the conversion rate to
;               something less than 1MHz.  We need two values depending on if
;               we are in test mode or not.
;               Test mode: 2.4MHz/4=1.6us x17=30us to convert
;               Regular mode: 8MHz/8=1us x17=17us to convert
; -----------------------------------------------------------------------------
a2dini:
$IF DEBUG
        mov     #$50,adclk      ; Bus clock source (2.4MHz) divided by 4
$ELSEIF
        mov     #$70,adclk      ; Bus clock source (8MHz) divided by 8
$ENDIF
        rts                     ; That's it!

        page
; -----------------------------------------------------------------------------
; Routine:      a2dc1w
; Called By:    timirq (iqothr)
; Calls:        None
; Gazintas:     None
; Gazoutas:     a2d1w 1-wire bus voltage (Full Scale=5V)
; Function:     This routine temporarily converts PB1 to an analog input and
;               converts the input voltage.  We have to do a dummy conversion
;               first as the first conversion after standby is garbage.  After
;               converting the needed value, we put the A/D back into standby.
;
;               Each conversion takes about 30us in debug mode and 17us in
;               regular mode.  With two conversions, this means this
;               routine takes about 60us in debug mode and 35us in
;               regular mode
; -----------------------------------------------------------------------------
a2dc1w: mov     #$00,adscr      ; fire off a dummy conversion
a2dlp1: brclr   7,adscr,a2dlp1  ; wait for it to complete

        mov     #$00,adscr      ; fire the 1-wire bus conversion (PB0)
a2dlp2: brclr   7,adscr,a2dlp2  ; wait for it to complete
        lda     adr             ; get bus voltage
        sta     a2d1w           ; save it for later

        mov     #$1f,adscr      ; turn the A/D off
        rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine:      a2dcbt
; Called By:    timirq (iqothr)
; Calls:        None
; Gazintas:     None
; Gazoutas:     a2dbty Battery voltage (Full-scale=15V)
; Function:     This routine temporarily converts PB7 to an analog input and
;               converts the input voltage.  We have to do a dummy conversion
;               first as the first conversion after standby is garbage.  After
;               converting the needed value, we put the A/D back into standby.
;               The conversion routines later assume that there is an analog
;               divide-by-three before the input.
;
;               Each conversion takes about 30us in debug mode and 17us in
;               regular mode.  With two conversions, this means this
;               routine takes about 60us in debug mode and 35us in
;               regular mode
; -----------------------------------------------------------------------------
a2dcbt: mov     #$07,adscr      ; fire off a dummy conversion
a2dlp3: brclr   7,adscr,a2dlp3  ; wait for it to complete

        mov     #$07,adscr      ; fire the battery voltage conversion (PB7)
a2dlp4: brclr   7,adscr,a2dlp4  ; wait for it to complete
        lda     adr             ; get battery voltage
        sta     a2dbty          ; save it for later

        mov     #$1f,adscr      ; turn the A/D off
        rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine:      a2dc5v
; Called By:    show3
; Calls:        bin2am
; Gazintas:     A: byte to convert
; Gazoutas:     matemp: ASCII conversion value (000-500)
; Function:     This routine takes the value in A and scales it from binary
;               0-256 to ASCII 000-500.  This is to take the A/D conversion
;               value and scale the conversion value to a 5.00V scale.
; -----------------------------------------------------------------------------
a2dc5v: ldx     #$fc            ; 1.953125 1/100ths of a volt per count (1.7 fmt)
        mul                     ; Answer in 9.7 format
        clr     mbcnv           ; clear MSB
        rola                    ; rotate LSByte up one to line up bytes
        rolx                    ; Rotate into middle byte
        stx     mbcnv+1         ; becomes LSB of displayed value
        rol     mbcnv           ; MSBit from middle byte for MSB in display value

        clra                    ; 0...
        sta     mzfill          ; want leading 0's
        jsr     bin2am          ; convert it
        rts

; -----------------------------------------------------------------------------
; Routine:      a2dc12
; Called By:    show3
; Calls:        bin2am
; Gazintas:     A: byte to convert
; Gazoutas:     matemp: ASCII conversion value (000-150)
; Function:     This routine takes the value in A and scales it from binary
;               0-256 to ASCII 000-150.  This is to take the A/D conversion
;               value and scale the conversion value to a 15.0V scale.
; -----------------------------------------------------------------------------
a2dc12: ldx     #150            ; 150/256 in 0.8 format
        mul                     ; x:a is answer in 8.8 format
        stx     mbcnv+1         ; Need only LSB for 0-128 range
        clr     mbcnv           ; Clear MSB

        clra                    ; 0...
        sta     mzfill          ; want leading 0's
        jsr     bin2am          ; convert it
        rts

