        page
******************************************************
** Copyright 2006 CMI Productions.                  **
** Lots of rights reserved.                         **
** Free for personal use as long as this copyright  **
** message is maintained in the source and the LCD  **
** copyright message is also preserved.  Commercial **
** use prohibited without CMIs written concent.     **
******************************************************
; -----------------------------------------------------------------------------
; File:         metric.asm
; Function:     Metric versions of the display routines
; Routines:     shomet: Metric launch routine
;               show0m: Current status
;               show1m: Todays stats
;               show2m: Yesterdays stats
;               show3m: Rain stats
;           N/A show4m: Internal status (nuthin metric)
;           N/A show5m: Error counters (nuthin metric)
; -----------------------------------------------------------------------------


; -----------------------------------------------------------------------------
; Routine:      shomet, show0m, show1m, sho3m
; Called By:    showit
; Calls:        wrmess (write message to LCD)
;               wrcmd (write LCD command byte)
;               wrdat (write LCD data byte)
;               pitemp (convert binary to ASCII temperature)
;               piwdir (convert binary to ASCII wind direction)
;               piwind (convert binary to ASCII wind speed)
; Gazintas:     cdisp (Current display mode)
;               errtmp (error flag/counter for temperature)
;               errdir (error flag/counter for wind direction)
;               errspd (error flag/counter for wind speed)
; Gazoutas:     none
; Function:     This is the display code.  This code takes the current data
;               set and presents it on the display.  We have more data that
;               can fit on the display at once, so multiple modes are
;               supported.
;               Note that data is only displayed if it is valid data.  If
;               errors occur during the sampling, the information is blanked
;               out on the display.
; -----------------------------------------------------------------------------
shomet: lda     cdisp           ; get the display number
        beq     show0m          ; if 0, do display #0
        deca                    ; see if 1
        beq     lsho1m          ; if so, do display #1
        deca                    ; see if 1
        beq     lsho2m          ; if so, do display #2
        deca                    ; see if 1
        beq     lsho3m          ; if so, do display #3
        deca                    ; see if 1
        beq     lsho4m          ; if so, do display #4
        deca                    ; see if 1
        beq     lsho5m          ; if so, do display #5
        bra     show0m          ; if anything else, default to 0

lsho1m: jmp     show1m          ; go do it.
lsho2m: jmp     show2m          ; go do it.
lsho3m: jmp     show3m          ; go do it.
lsho4m: jmp     show4           ; go do it. (non-metric)
lsho5m: jmp     show5           ; go do it. (non-metric)

; -----------------------------------------------------------------------------
;       Display mode 0: Current weather
;
;        --------------------
;       |     Currently      |
;       |Tmp xxxoC Rain x.xxm| Current temperature and rainfall since midnight
;       |Wnd xxxo xxKPH G xx | Current wind direction, speed, 5-min gust
;       |RH xx% DPxxC Bpxxxx | Relative humidity and dewpoint, BP, and trend
;        --------------------
; -----------------------------------------------------------------------------

show0m: lda     #6              ; Big message #6 (status page 0 metric)
        jsr     wr4mes          ; write it
        clra                    ; 0
        sta     mzfill          ; for this stuff, dont' zero-fill

        brset   7,errtmp,sho0ma ; if error, don't show temperature
        lda     ddtemp          ; get current temperature
        sub     #110            ; convert to two's comp
        sta     macnv           ; place to get converted
        jsr     pntemp          ; convert current temp to degrees farenheit
        lda     #$c4            ; Line 2, Col 5 (temperature)
        jsr     sho3dg          ; show temperature

sho0ma: lda     rirgot          ; see if any rain guage present
        bne     sho0mb          ; if not, leave all entries blank
        brset   7,errran,sho0mb ; if sample error, skip current rain stat

        lda     #1              ; not 0
        sta     mzfill          ; want leading 0's on rainfall
        ldhx    rantdy          ; get todays rain
        sthx    mbcnv           ; store for conversion
        jsr     pnrain          ; go convert it to ASCII
        lda     #$cf            ; todays rain location (line 4, col 5)
        jsr     shmran          ; show the rain

sho0mb: brset   7,errdir,sho0mc ; if direction error, don't show data
        lda     fdmode          ; get users wind direction display mode
        jsr     pmwdir          ; convert wind direction to degrees
        lda     #$98            ; Line 3, Col 5 (wind direction)
        jsr     sho3dg          ; show it

        lda     fdmode          ; check wind direction display mode
        beq     sho0mc          ; if numeric, no action required
        lda     #' '            ; if text direction erase degree symbol
        jsr     wrdat           ; do it

sho0mc: lda     errspd          ; get wind speed error counter
        and     #$c0            ; check both bits
        bne     sho0md          ; if any errors, don't show
        clra                    ; 0
        sta     mzfill          ; for this stuff, dont' zero-fill
        lda     winspd          ; get current wind speed MSB
        sta     macnv           ; conversion buffer
        lda     winspd+1        ; get cyrrebt wind speed LSB
        sta     macnv+1         ; conversion buffer
        jsr     pnwind          ; convert 5-sec value to KPH
        lda     #$9d            ; Line 3, Col 10 (wind speed)
        jsr     sho2dg          ; show it

sho0md: lda     maxws           ; get max wind speed MSB
        sta     macnv           ; conversion buffer
        lda     maxws+1         ; get max wind speed LSB
        sta     macnv+1         ; conversion buffer
        jsr     pnwind          ; convert 5-sec value to MPH
        lda     #$a5            ; Line 3, Col 18 (gust speed)
        jsr     sho2dg          ; show it

sho0me: lda     fihum           ; get humidity sensor ID
        cmp     #$26            ; see if programmed
        bne     sho0mg          ; skip if no sensor programmed
        brset   7,errhum,shomx0 ; if humidity error, skip too

        clra                    ; 0...
        sta     mzfill          ; No leading 0's on humidity
        clr     mbcnv           ; conversion buffer MSB
        lda     hmval           ; get current humidity value
        and     #$7f            ; clear excess flag
        sta     mbcnv+1         ; conversion buffer
        jsr     bin2am          ; convert to ASCII
        lda     #$d6            ; Line 4, Col 3
        jsr     sho3dg          ; show it

sho0mf: lda     hmdew           ; get current temperature
        sub     #110            ; convert to two's comp
        sta     macnv           ; place to get converted
        jsr     pntemp          ; convert current temp to degrees farenheit
        lda     #$dd            ; Line 4, Col 10
        jsr     sho3dg          ; show temperature

sho0mg: lda     fibar           ; get barometric sensor id
        cmp     #$26            ; see if progammed
        bne     shomx0          ; skip if no sensor programmed
        lda     errbar
        cmp     #$80            ; see if hi bit set indicating error
        bhs     shomx0          ; if barometric error, shipto top

        jsr     bab2m           ; convert baro. sensor to ASCII metric
        lda     #lcd4+16        ; Line 4, Col 16

        jsr     wrcmd           ; set it.
        lda     matemp          ; send first digit to LCD
        jsr     wrdat           ; write it to screen
        lda     matemp+1        ; send second digit to a reg
        jsr     wrdat           ; write to screen
        lda     matemp+2        ; Bp digit 3
        jsr     wrdat
        lda     matemp+3        ; get last digit
        jsr     wrdat

        lda     #lcd4+19        ; last character on last line
        jsr     wrcmd           ; set it
        jsr     bardel          ; see if baro. is rising, falling or steady
        jsr     wrdat           ; write message

shomx0: rts                     ; all done

        page
; -----------------------------------------------------------------------------
;       Display mode 1: Todays stats
;
;        --------------------
;       |    Todays Stats    |
;       |Temp Hi xxx Lo xxx  | Todays high, low temperature
;       |Max Gust xxKPH      | Todays max wind gust
;       |Rain x.xxm Bp ----Mb  Todays rainfall
;        --------------------
; -----------------------------------------------------------------------------

show1m: lda     #13             ; Big message #13 (metric status page 1)
        jsr     wr4mes          ; write it
        clra
        sta     mzfill          ; for this stuff, dont' zero-fill

shom1a: lda     temphi          ; get current hi temp
        cmp     #$ff            ; see if un-initialized
        beq     shom1b          ; if so, leave dashes
        sub     #110            ; convert to two's comp
        sta     macnv           ; place to get converted
        jsr     pntemp          ; convert low temp to degrees farenheit
        lda     #$c8            ; Line 2, Col 12 (low temperature)
        jsr     sho3dg          ; show it

shom1b: lda     templo          ; get current high temp
        cmp     #$ff            ; see if un-initialized
        beq     shom1c          ; if so, leave dashes
        sub     #110            ; convert to two's comp
        sta     macnv           ; place to get converted
        jsr     pntemp          ; convert high temp to degrees farenheit
        lda     #$cf            ; Line 2, Col 17 (high temperature)
        jsr     sho3dg          ; show it

shom1c: lda     todygs          ; get todays gust speed
        sta     macnv           ; conversion buffer
        lda     todygs+1        ; get current wind speed MSB
        sta     macnv+1         ; conversion buffer
        jsr     pnwind          ; convert 5-sec value to KPH
        lda     #$9d            ; Line 4, Col 19
        jsr     sho2dg          ; show it

        lda     rirgot          ; see if any rain guage present
        bne     shom1d          ; if not, leave all entries blank
        brset   7,errran,shom1d ; if sample error, skip current rain stat

        lda     #1              ; not 0
        sta     mzfill          ; want leading 0's on rainfall
        ldhx    rantdy          ; get todays rain
        sthx    mbcnv           ; store for conversion
        jsr     pnrain          ; go convert it to ASCII
        lda     #$d9            ; todays rain location (line 4, col 5)
        jsr     shoran          ; show the rain

        ; Add bar conversion and display to code here

shom1d: nop                     ; space waster
shomx1: rts                     ; all done

        page
; -----------------------------------------------------------------------------
;       Display mode 2: Yesterdays stats
;        --------------------
;       |     Yesterday      |
;       |Temp L---oC H---oC  | High and Low temp
;       |Wind Gust -- KPH    | Wind gust
;       |Rain --.-m          | Rainfall
;        --------------------
; -----------------------------------------------------------------------------

show2m: lda     #7              ; Big message #7 (status page 1 metric)
        jsr     wr4mes          ; write it
        clra                    ; 0
        sta     mzfill          ; for this stuff, dont' zero-fill

        lda     yestlo          ; get yesterdays high temperature
        cmp     #$ff            ; see if valid
        beq     sho2mh          ; if not, skip it
        sub     #110            ; convert to two's comp
        sta     macnv           ; place to get converted
        jsr     pntemp          ; convert low temp to degrees farenheit
        lda     #$c6            ; Line 2, Col 7
        jsr     sho3dg          ; show it

sho2mh: lda     yesthi          ; get yesterdays low temperature
        cmp     #$ff            ; see if valid
        beq     sho2ml          ; if not, skip it
        sub     #110            ; convert to two's comp
        sta     macnv           ; place to get converted
        jsr     pntemp          ; convert low temp to degrees farenheit
        lda     #$cd            ; Line 2, Col 14
        jsr     sho3dg          ; show it

sho2ml: lda     yestgs          ; get Yesterdays gust speed
        sta     macnv           ; conversion buffer
        lda     yestgs+1        ; get current wind speed MSB
        sta     macnv+1         ; conversion buffer
        jsr     pnwind          ; convert 5-sec value to MPH
        lda     #$9e            ; Line 2, Col 11
        jsr     sho2dg          ; show it

        lda     rirgot          ; see if any rain guage present
        bne     shomx2          ; if not, leave all entries blank

        lda     #1              ; not 0
        sta     mzfill          ; want leading 0's on rainfall
        ldhx    ranyst          ; get yesterdays rain
        sthx    mbcnv           ; store for conversion
        jsr     pnrain          ; go convert it to ASCII
        lda     #$d9            ; yesterdays rain location (row 4, col 5)
        jsr     shmran          ; show the rain

shomx2: rts                     ; all done

        page
; -----------------------------------------------------------------------------
;       Display mode 3: All the rain stats
;        --------------------
;       |       Rain         |
;       |Today --.- Yest --.-| Todays (so far) and yesterdays rain
;       |Month --.- User --.-| Month (so far) and user-ref rain
;       |Hour  --.-          | Last hour
;        --------------------
; -----------------------------------------------------------------------------

show3m: lda     #3              ; big message #3 (status #2)
        jsr     wr4mes          ; write it
        lda     #1              ; not 0
        sta     mzfill          ; want leading 0's on rainfall

        lda     rirgot          ; see if any rain guage present
        bne     sho3mx          ; if not, leave all entries blank
        brset   7,errran,sho3mc ; if sample error, skip current rain stat

        ldhx    rantdy          ; get todays rain
        sthx    mbcnv           ; store for conversion
        jsr     pnrain          ; go convert it to ASCII
        lda     #$c6            ; todays rain location
        jsr     shmran          ; show the rain

sho3mc: ldhx    ranyst          ; get yesterdays rain
        sthx    mbcnv           ; store for conversion
        jsr     pnrain          ; go convert it to ASCII
        lda     #$d0            ; yesterdays rain location
        jsr     shmran          ; show the rain

        brset   7,errran,sho3mx ; if sample error, skip current rain stat

        lda     rancnt+1        ; get current rain counter LSB
        sub     ranmth+1        ; subtract reference LSB
        sta     mbcnv+1         ; save result
        lda     rancnt          ; get current rain counter MSB
        sbc     ranmth          ; subtract reference MSB
        sta     mbcnv           ; save result
        jsr     pnrain          ; go convert it to ASCII
        lda     #$9a            ; cumulative rain location
        jsr     shmran          ; show the rain

        lda     rancnt+1        ; get current rain counter LSB
        sub     firref+1        ; subtract reference LSB
        sta     mbcnv+1         ; save result
        lda     rancnt          ; get current rain counter MSB
        sbc     firref          ; subtract reference MSB
        sta     mbcnv           ; save result
        jsr     pnrain          ; go convert it to ASCII
        lda     #$a4            ; cumulative rain location
        jsr     shmran          ; show the rain

        ldhx    ran1hr          ; get rain past hour
        sthx    mbcnv           ; save it for conversion
        jsr     pnrain          ; go convert it to ASCII
        lda     #$da            ; hour's worth rain location
        jsr     shmran          ; show the rain

sho3mx: rts                     ; all done

