        page
******************************************************
** Copyright 2006 CMI Productions.                  **
** Lots of rights reserved.                         **
** Free for personal use as long as this copyright  **
** message is maintained in the source and the LCD  **
** copyright message is also preserved.  Commercial **
** use prohibited without CMIs written concent.     **
******************************************************
; -----------------------------------------------------------------------------
; File:         setup.asm
; Function:     Operations setup routines.
; Routines:     stime:  Sets up the real-time clock in GMT and local time
;               ssens:  Sets up the temperature/wind sensor.
;               sopts:  Sets up the UART/Display options.
;               sposn:  Sets up the Position string and MX614 parameters
;               rdf64:  Read 64 bytes from flash to (msg)
;               wrf64:  write 64 bytes from (msg) to flash
;               getext: Get a line of ASCII data from the user
;               spssid: Get an SSID (0-15) from the user
;               spsssi: Display the SSID
;               getrky: Get key with repeat-on-hold for up/down
;               setldy: Set 1-digit delay value
; -----------------------------------------------------------------------------

; -----------------------------------------------------------------------------
; Routine:      stime
; Called By:    menus
; Calls:        wrmess (write message to LCD)
;               wrcmd (write command byte to LCD)
;               bin2as (convert binary to ASCII)
;               wrstr (write string to LCD)
;               mthmax (figger max days in month)
; Gazintas:     None
; Gazoutas:     month,day,year,hours,mins,secs,lhrs (new time)
; Function:     This is the routine to set the UTC time and local time.  It is
;               used for the header of the weather string.  While setting the
;               time, we turn off the time part of the timer (rtchld) so that
;               the time doesn't increment while we are adjusting the time.
;               Local time is so we can reset the 24-hour stats at midnight.
; -----------------------------------------------------------------------------
stime:  lda     #1              ; not 0
        sta     zfill           ; want zero-filling

        lda     #18             ; message #18 (setting time)
        jsr     wrmess          ; write it
        clr     keycnt          ; clear key counter
        inc     rtchld          ; hold off RTC
        lda     #$0e            ; cursor on command
        jsr     wrcmd           ; turn the cursor on
        clr     cntr            ; our field pointer

stlp:   lda     cntr            ; get the field counter
        cmp     #5              ; see if we are at the last field
        bne     stlp14          ; if not lst one, screen 1-4 processing

stlp5:  lda     #20             ; Set Local Hour message
        jsr     wrmess          ; show message
        lda     #$8f            ; the place for the local hour
        sta     msg             ; start of message

        lda     lhrs            ; get the local hours
        sta     divdnd+1        ; place to convert
        clr     divdnd          ; clear MSB
        jsr     bin2as          ; convert it
        lda     atemp+1         ; get 10's
        sta     msg+1           ; store it
        lda     atemp+2         ; get 1's
        sta     msg+2           ; store it
        clr     msg+3           ; EOS flag
        jmp     stmsg           ; go write it


stlp14: lda     #$86            ; the place to write the string to
        sta     msg             ; spot #1
        lda     #'/'            ; date seperator
        sta     msg+3           ; spot #1
        sta     msg+6           ; spot #2
        lda     #' '            ; space between date and time
        sta     msg+9           ; plop it down
        lda     #':'            ; colon between hrs and mins
        sta     msg+12          ; plop it down
        clr     msg+15          ; the EOS chracter

        lda     month           ; get the month
        sta     divdnd+1        ; place to convert
        clr     divdnd          ; clear MSB
        jsr     bin2as          ; convert it
        lda     atemp+1         ; get 10's
        sta     msg+1           ; store it
        lda     atemp+2         ; get 1's
        sta     msg+2           ; store it

        lda     day             ; get the day
        sta     divdnd+1        ; place to convert
        clr     divdnd          ; clear MSB
        jsr     bin2as          ; convert it
        lda     atemp+1         ; get 10's
        sta     msg+4           ; store it
        lda     atemp+2         ; get 1's
        sta     msg+5           ; store it

        lda     year            ; get the year
        sta     divdnd+1        ; place to convert
        clr     divdnd          ; clear MSB
        jsr     bin2as          ; convert it
        lda     atemp+1         ; get 10's
        sta     msg+7           ; store it
        lda     atemp+2         ; get 1's
        sta     msg+8           ; store it

        lda     hrs             ; get the hour
        sta     divdnd+1        ; place to convert
        clr     divdnd          ; clear MSB
        jsr     bin2as          ; convert it
        lda     atemp+1         ; get 10's
        sta     msg+10          ; store it
        lda     atemp+2         ; get 1's
        sta     msg+11          ; store it

        lda     mins            ; get the minutes
        sta     divdnd+1        ; place to convert
        clr     divdnd          ; clear MSB
        jsr     bin2as          ; convert it
        lda     atemp+1         ; get 10's
        sta     msg+13          ; store it
        lda     atemp+2         ; get 1's
        sta     msg+14          ; store it

stmsg:  ldhx    #msg            ; the string to write
        jsr     wrstr           ; go write it
        ldx     cntr            ; our field pointer
        clrh                    ; make sure H is 0
        lda     stcol,x         ; get the place to put the cursor
        jsr     wrcmd           ; place it

stscan: lda     curkey          ; get key pressed (if any)
        cmp     #kwait          ; see if wait code
        beq     stscan          ; no new data yet, wait
        cmp     #ksel           ; see if select
        beq     lmenus          ; if so, off to menus
        cmp     #knone          ; if no key, reset rapid count
        beq     strst           ; go do it
        cmp     #kup            ; see if up key
        beq     stupdn          ; go process
        cmp     #kdown          ; see if down key
        beq     stupdn          ; go do it if so

strst:  clr     keycnt          ; clear key pressed count
        bra     stscan          ; go wait for a key

lmenus: jmp     llmenu          ; long jump

stupdn: lda     #1              ; default inc/dec by 1
        sta     temp            ; remember it
        lda     keycnt          ; get key counter
        beq     stdoit          ; if 0 value, go inc/dec by 1
        cmp     #11             ; see if 11 samples
        blo     stkwt           ; if 1-11, no action
        dec     keycnt          ; don't want keycnt to overflow
        lda     #5              ; inc/dec by 5 when held down
        sta     temp            ; remember it

stdoit: lda     curkey          ; get keycode back again
        cmp     #kdown          ; see if down (vs up)
        beq     stdown          ; if so, go decrement time

stup:   ldx     cntr            ; get our counter
        clrh                    ; make sure H is 0
        ldx     stoff,x         ; point to real variable
        inc     year,x          ; get the field to increment
        lda     year,x          ; get incremented value
        cpx     #2              ; see if day of month
        bne     stupnf          ; if not, normal processing

        jsr     mthmax          ; get max days in current month
        sta     temp            ; save it
        ldx     cntr            ; get our counter again (X trashed)
        clrh                    ; make sure H is 0
        ldx     stoff,x         ; point to real variable
        lda     year,x          ; get it
        cmp     temp            ; do compare
        bra     stupfc          ; do checking

stupnf: cmp     stmax,x         ; compare to max value
stupfc: bls     stntim          ; if within limits, all done
        lda     stmin,x         ; otherwise roll, get min value
        sta     year,x          ; save as new value
        bra     stntim          ; all done

stdown: ldx     cntr            ; get our pointer
        ldx     stoff,x         ; point to real variable
        clrh                    ; make sure H is 0
        dec     year,x          ; get the field to increment
        lda     year,x          ; get incremented value
        bmi     stdnnn          ; if negative, we have underflowed
        cmp     stmin,x         ; compare to max value
        bhs     stntim          ; if within limits, all done
stdnnn: lda     stmax,x         ; otherwise roll, get min value
        sta     year,x          ; save as new value
        cpx     #2              ; see if day of month
        bne     stntim          ; if not, normal processing

        jsr     mthmax          ; get max days in current month
        ldx     cntr            ; get our counter again (X trashed)
        ldx     stoff,x         ; point to real variable
        clrh                    ; make sure H is 0
        sta     year,x          ; do compare

stntim: inc     keycnt          ; increment our key counter
        lda     #kwait          ; wait for next scan flag
        sta     curkey          ; store it
lstlp:  jmp     stlp            ; go display new time

stkwt:  inc     keycnt          ; increment key count
        lda     #kwait          ; wait for next scan flag
        sta     curkey          ; store it
        jmp     stscan          ; wait for next scan

llmenu: lda     curkey          ; get current keycode
        cmp     #knone          ; see if no key
        bne     llmenu          ; wait for no key
        inc     cntr            ; if select pressed, increment counter
        lda     cntr            ; get incremented value
        cmp     #6              ; see if done
        bne     lstlp           ; if not, loop back, else done

; Now that the clock is set, we need to actually remember one thing,
; that is the offset between GMT and local time.  This is needed in case
; the user has the DS1994.  That way we can automatically set up both
; local and GMT time on power-up if the DS1994 is present.  Rather than
; take up a whole 128-byte block for this little offset, we share with
; the options area by reading everything in, erasing the entire block,
; then writing it all back again.

        ldhx    #fpage2         ; the page to read
        jsr     rdf64           ; read it

        lda     hrs             ; get GMT hours
        sub     lhrs            ; subtract local hours for offset
        sta     msg+3           ; save result for flash

        sei                     ; interrupts off
        ldhx    #fpage2         ; point to whole page again
        jsr     appera          ; erase it

        ldhx    #fpage2         ; point to whole page again
        jsr     wrf64           ; program altered values
        cli                     ; interrupts back on

        clr     rtchld          ; re-enable RTC
        lda     #$0c            ; cursor off command
        jsr     wrcmd           ; turn the cursor off
        jmp     menus           ; back to menus

        page
; -----------------------------------------------------------------------------
; Routine:      sopts
; Called By:    menus
; Calls:        wrmess (write message to LCD)
;               waits (wait in 1/10th second increments)
;               appera (application-called flash erase)
;               apppgm (application-called flash program)
; Gazintas:     None
; Gazoutas:     fumode (UART mode)
;               fmmode (Metric mode)
;               fdmode (Direction display mode)
;               fbaud  (UART mode and baud rate)
; Function:     This is the routine that goes through all the miscellaneous
;               modes of the weather station.  These are all lumped together
;               so that they can be erased and reprogrammed all at the same
;               time.
;               fumode: UART mode:
;                 0  - TNC direct mode 4800 baud, 5-min averaged data
;                 1  - TNC LTP mode    2400 baud, 5-min averaged data
;                 2  - Computer mode   9600 baud, 5-sec live data
;                 3  - Peet Mode       2400 baud, 5-sec live data
;                 4+ - Modem Mode      1200 baud, 5-min averaged data
;               fdmode: Display mode
;                 0  - Show degrees for wind direction on LCD
;                 1+ - Show compass points for wind direction on LCD
;               fmmode: Metric flag
;                 0  - Metric units
;                 1+ - English units
;               fbaud: baud rate setting
;                 0  - 2400
;                 1  - 4800
;                 2  - 9600
;                 3+ - reset to default
;
; -----------------------------------------------------------------------------
sopts:  lda     #0              ; instructions
        jsr     wrmess          ; display them
        lda     #40             ; 4-second delay
        jsr     waits           ; wait it

        lda     fumode          ; get UART mode byte
        cmp     #04             ; compare to max value
        bls     sopfok          ; if OK, no filtering needed
        clra                    ; if not, assume 0
sopfok: sta     mndray          ; save in RAM

        lda     fdmode          ; get display flag
        and     #$01            ; filter it
        sta     mndray+1        ; save in RAM

        lda     fmmode          ; get metric flag
        and     #$01            ; filter it
        sta     mndray+2        ; save in RAM

        lda     ftoff           ; get local-to-GMT offset
        sta     mndray+3        ; save in RAM

        lda     fbaud           ; get baud rate setting
        and     #$03            ; filter it
        sta     mndray+4        ; save in RAM

; -----------------------------------------------------------------------------
;       UART mode first...
; -----------------------------------------------------------------------------

suarlp: lda     mndray          ; get the UART mode
        add     #10             ; offset to message #
        jsr     wrmess          ; show it
suwait: jsr     wfnkey          ; get the current key
        cmp     #ksel           ; see if select button
        beq     sbrate          ; if so, go do that menu
        cmp     #kup            ; see if up key
        bne     suup            ; if so, increment menu item
        cmp     #kdown          ; see if down key
        bne     sudn            ; if so, decrement menu item
        bra     suwait          ; if none of above, wait for real key

        ; Whenever the UART mode is changed, set the UART baud
        ; rate to AUTO.

sudn:   lda     #3              ; auto baud rate
        sta     mndray+4        ; make it so
        lda     mndray          ; get menu item number
        deca                    ; decrement it
        bpl     suudun          ; if positive, all done
        lda     #4              ; otherwise wrap to 4
        bra     suudun          ; go post-process it

suup:   lda     #3              ; auto baud rate
        sta     mndray+4        ; make it so
        lda     mndray          ; get menu item number
        inca                    ; increment it
        cmp     #5              ; see if we went too far
        bne     suudun          ; if not, go save
        clra                    ; otherwise back to 0
suudun: sta     mndray          ; save it
        bra     suarlp          ; go show next menu

; -----------------------------------------------------------------------------
;       Sneak in baud rate after UART mode
; -----------------------------------------------------------------------------

sbrate: lda     mndray          ; get mode back
        cmp     #4              ; see if modem mode
        bne     sbrnmd          ; if not, normal processing

        mov     #1,mndray+4     ; fix baud rate at 4800
        bra     sdmlp           ; on to LCD mode

sbrnmd: lda     mndray+4        ; get baud rate (or auto if 1st time)
        cmp     #3              ; see if flagged for auto
        bne     sbnau           ; if not, leave as-is
        ldx     mndray          ; otherwise get UART mode
        clrh                    ; H=0
        lda     brdtbl,x        ; get default baud rate
        sta     mndray+4        ; and make it the choice

sbnau:  lda     #39             ; Baud Rate message
        jsr     wrmess          ; write it

sbrlp:  lda     mndray+4        ; get the baud rate
        ldx     #6              ; 6 chars per entry
        mul                     ; get offset
        add     #brttbl%256     ; get message #0 address LSB
        sta     addrlo          ; save LSB

        txa                     ; get offset MSB
        adc     #brttbl\256     ; get message #0 MSB
        sta     addrhi          ; save MSB

	ldhx	addrhi		; get full 16-bit address in hx
        jsr	wrstr		; go write the baud rate string

        ; Now that we have displayed the current value, time to let the user
        ; set something else. All baud rates are available in all modes now.

sbwait: jsr     wfnkey          ; get the current key
        cmp     #ksel           ; see if select button
        beq     sdmlp           ; if so, go do that menu
        cmp     #kup            ; see if up key
        bne     sbup            ; if so, increment menu item
        cmp     #kdown          ; see if down key
        bne     sbdn            ; if so, decrement menu item
        bra     sbwait          ; if none of above, wait for real key

sbdn:   lda     mndray+4        ; get menu item number
        deca                    ; decrement it
        bpl     sbudun          ; if OK, done

sbdno:  lda     #2              ; otherwise wrap to 2 (9600)
        bra     sbudun          ; go post-process it

sbup:   lda     mndray+4        ; get menu item number
        inca                    ; increment it
        cmp     #3              ; see if we went too far
        bne     sbudun          ; if not, go save
        clra                    ; otherwise restart with 2400

sbudun: sta     mndray+4        ; save it
        bra     sbrlp           ; go show result and wait

; -----------------------------------------------------------------------------
;       LCD wind direction display mode...
; -----------------------------------------------------------------------------

sdmlp:  lda     mndray+1        ; get the display mode
        add     #28             ; offset to message #
        jsr     wrmess          ; show it
sudwt:  jsr     wfnkey          ; get the current key
        cmp     #ksel           ; see if select button
        beq     smmlp           ; if so, go do that menu
        cmp     #kup            ; see if up key
        bne     sudup           ; if so, increment menu item
        cmp     #kdown          ; see if down key
        bne     suddn           ; if so, decrement menu item
        bra     sudwt           ; if none of above, wait for real key

suddn:  lda     mndray+1        ; get menu item number
        deca                    ; decrement it
        bra     suddun          ; go post-process it

sudup:  lda     mndray+1        ; get menu item number
        inca                    ; increment it
suddun: and     #$01            ; truncate to 0-1
        sta     mndray+1        ; save it
        bra     sdmlp           ; go show next menu

; -----------------------------------------------------------------------------
;       Metric/English mode select...
; -----------------------------------------------------------------------------

smmlp:  lda     mndray+2        ; get the units mode
        add     #23             ; offset to message #
        jsr     wrmess          ; show it
smmwt:  jsr     wfnkey          ; get the current key
        cmp     #ksel           ; see if select button
        beq     sudun           ; if so, on to next option
        cmp     #kup            ; see if up key
        bne     smmud           ; if so, increment menu item
        cmp     #kdown          ; see if down key
        bne     smmud           ; if so, decrement menu item
        bra     smmwt           ; if none of above, wait for real key

smmud:  lda     mndray+2        ; get menu item number
        inca                    ; increment it (up or down doesn't matter)

smmdun: and     #$01            ; truncate to 0-1
        sta     mndray+2        ; save it
        bra     smmlp           ; go show next menu

; -----------------------------------------------------------------------------
;       Wrap-up, write variables back to Flash.
; -----------------------------------------------------------------------------

sudun:  sei                     ; stop interrupts
        ldhx    #fpage2         ; page to erase
        jsr     appera          ; go earse it
        lda     #$ff            ; un-initialized value
        sta     mndray+5        ; clear unused bytes
        sta     mndray+6        ; clear unused bytes
        sta     mndray+7        ; clear unused bytes
        ldhx    #fpage2         ; point to mode bytes
        jsr     apppgm          ; go program them
        cli                     ; restart interrupts
        jsr     mxpten          ; reconfigure IOs if needed
        jmp     menus           ; go back to menus

        page
; -----------------------------------------------------------------------------
; Routine:      ssens
; Called By:    menus
; Calls:        i_rst  (do a 1-wire bus reset
;               wrmess (write message to LCD)
;               wrcmd  (write a command byte to the LCD)
;               wrdat  (write a data byte to the LCD
;               wrline (write a line to the LCD
;               waits  (wait in 1/10th second increments)
;               wfnkey (wait for no key pressed)
;               appera (application-called flash erase)
;               apppgm (application-called flash program)
;               iswitch (turn on/off wind direction switch)
;               isrch  (search for a wind directio ID)
;               amatch (see if wind direction match in table)
;               rddir  (read current wind direction)
;               cnvid  (convert sensor ID to ASCII at (msg+1)
; Gazintas:     None
; Gazoutas:     fitemp (current temperature sensor address)
;               fisw (current wind directio switch address)
;               ficntr (current wind speed counter address)
;               fidir1-8 (eight wind direction sensor addresses)
;               fwdoff (wind direction 0-degree offset)
; Function:     This is the routine to go find out all the sensor IDs for the
;               main weather unit that does temperature, wind speed, and wind
;               direction.
;
;               Temperature invloves going out and seeing if there is more
;               than one temperature sensor out there.  If there is, the user
;               is asked which one to use.
;
;               For wind speed the same sensor ID is used as the rain guage.
;               If two such sensors are detected, I ask the user to unplug the
;               rain guage before proceding.  This isn't so hot and needs to
;               be made easier.
;
;               This is the routine to calibrate the wind direction sensors.
;               This is a pretty nasty chore as we do not know ahead of time
;               anything about the eight direction sensors.  The first step is
;               to find out the IDs of the eight sensors.  We do this by
;               asking the user to rotate the wind vane clockwise
;               (incrementing degrees).  When the user starts the rotations,
;               we first try and find eight unique serial numbers.  If we get
;               any repeats before these eight unique numbers, we assume the
;               user screwed up and start over again.  While we are getting
;               these new IDs, we show the numbers 1-8 on the display along
;               with the text 1/3 indicating we are working on step 1 of three.
;               Step two happens after we get the eighth (and last)consecutive
;               new ID.  We change the counter to show 2/3 and expect to see
;               all eight IDs we got in step 1 to happen again in sequence.
;               If this fails, we go back to step 1.  Step three is the same
;               as step two, is probably not necessary, but I want to do it
;               anyway.
;
;               The final step is to tell the person to point the weather
;               vane north (as if the wind was coming from the north), and
;               press select.  This establishes the internal offset so we know
;               what direction is north.
; -----------------------------------------------------------------------------

ssens:  brclr   0,portb,serrl   ; If 1-wire bus is low, we got a problem
        jsr     i_rst           ; issue a 1-wire reset
        beq     ssbok           ; if a response, to on to cal's

serrh:  jmp     lerrh           ; long jump to bus stuck high (in wx08.asm)
serrl:  jmp     lerrl           ; long jump to bus stuck low (in wx08.asm)
sclex:  jmp     menus           ; long jump out

ssbok:  lda     #8              ; message #8 (Instructions)
        jsr     wr4mes          ; write it
        lda     #40             ; four second wait
        jsr     waits           ; wait it

        clr     menun           ; clear menu number

ssslp:  lda     #40             ; message base
        add     menun           ; get actual message number
        jsr     wrmess          ; write it

sswait: jsr     wfnkey          ; get the current key
        cmp     #ksel           ; see if select button
        beq     ssgo            ; if so, go do that menu

ssnext: inc     menun           ; otherwise increment selection
        lda     menun           ; get it
        cmp     #6              ; see if end
        bne     ssslp           ; if not, on to next menu
        bra     sclex           ; if end, all done

ssgo:   lda     curkey          ; get current keypress (select)
        bne     ssgo            ; wait for select to go away
        lda     menun           ; get the menu number
        beq     clltmp          ; 0=temperature cal
        deca                    ; check for menu 1
        beq     cllwsp          ; 1=wind speed cal
        deca                    ; check for menu 2
        beq     cllwdr          ; 2=wind direction cal
        deca                    ; check for menu 3
        beq     cllran          ; 3=rain gauge cal
        deca                    ; check for menu 4
        beq     cllhum          ; 4=humidity cal
        deca                    ; check for menu 5
        beq     cllbar          ; 5=barometer cal
        bra     ssens           ; if none, start over

clltmp: jsr     caltmp          ; temperature sensor cal
        bra     ssnext          ; next menu

cllwsp: jsr     calwsp          ; temperature sensor cal
        bra     ssnext          ; next menu

cllwdr: jsr     calwdr          ; temperature sensor cal
        bra     ssnext          ; next menu

cllran: jsr     calran          ; temperature sensor cal
        bra     ssnext          ; next menu

cllhum: jsr     calhum          ; temperature sensor cal
        bra     ssnext          ; next menu

cllbar: jsr     calbar          ; barometer cal
        bra     ssnext          ; next menu;
        page
; -----------------------------------------------------------------------------
; Routine:      sposn
; Called By:    menus
; Calls:        wrmess  (write message to LCD)
;               wrcmd   (write command byte to LCD)
;               wrstr   (write a string to the LCD)
;               appera  (erase data area)
;               apppgm  (program data area)
; Gazintas:     fiposn  (position string)
;               fmycal  (Station call sign)
;               fpath   (UI path)
; Gazoutas:     fiposn  (position string)
;               fmycal  (Station call sign)
;               fpath   (UI path)
; Function:     This is the routine to set the position string.  If the first
;               byte of the postion string is $ff, then no position string
;               is to be sent.  The postion string is a fixed length string
;               in the format ddmm.mmN/dddmm.mmW.  If enabled, this string
;               gets asserted between the time and weather data fields.
;               NOTE:  When position data is requested, the formats for both
;               time and weather get altered slight.  Weird, but the APRS
;               spec says so.  See the UART formatting routines for details.
;               Note that the IRQ-used msg buffer is available as we have
;               weather logging turnd off right now.
;               ------------------------------
;               With MX-614 support, this routine now also adds support for
;               the "mycall" parameter as well as "uipath" parameters
; -----------------------------------------------------------------------------
sposn:  ldhx    #fpage4         ; ID datablock
        jsr     rdf64           ; read the data

;       Before we get too carried away, we see if there is nothing at all
;       in the position string buffer.  If so, we initialize it a valid
;       ASCII sequence.  In either case we always re-initialize the fixed
;       bytes in case they got corrupted.

        lda     msg             ; get first byte of message
        cmp     #$ff            ; see if un-initialized
        bne     spsok           ; if not, normal processing

spoin:  ldx     #17             ; wanna initialize the whole message
        clrh                    ; MSB=0
        lda     #'0'            ; ASCII '0'
spsip:  sta     msg,x           ; save the byte
        decx                    ; decrement pointer
        bpl     spsip           ; do 'em all

        ldhx    #msg+63         ; last byte in the text buffers
spsclp: clr     ,x              ; clear a byte
        decx                    ; go back one
        cpx     #msg+18         ; see if we are in position string memory
        bne     spsclp          ; loop until so.

        mov     #'N',msg+7      ; default to North
        mov     #'W',msg+17     ; Default to West

spsok:  mov     #'.',msg+4      ; Longitude period
        mov     #'/',msg+8      ; slash between coords
        mov     #'.',msg+14     ; Latidute period

        lda     #4              ; message #4 (position string on/off)
        jsr     wrmess          ; write it

spoolp: ldhx    #offmsg         ; point to off message
        lda     msg+18          ; get the on/off flag
        cmp     #$ff            ; see if message off
        beq     spoff           ; if so, position string off
        ldhx    #onmsg          ; otherwise change to on message
spoff:  jsr     wrstr           ; go write the current status

        jsr     wfnkey          ; Wait for a new key press
        cmp     #ksel           ; see if select
        beq     spoock          ; if so, check answer
        cmp     #kmenu          ; see if menu button (no action)
        beq     spoolp          ; do nuthin

;       The simple thing to do here would be just to complement the flag, but
;       just in case it is something other than 00 or ff, I do explicit tests.

        tst     msg+18          ; For all other keys, test on/off flag
        beq     spmoff          ; if 0, want it off now
        clr     msg+18          ; otherwise clear it
        bra     spoolp          ; and wait for next key.

spmoff: mov     #$ff,msg+18     ; off flag
        bra     spoolp          ; wait for next key.

spoock: tst     msg+18          ; get on/off flag
        beq     spstr           ; if on, go do string
        jmp     spwrite         ; if off, skip string and write

        page
;       We get here if the message is turned on so the user can set the
;       message.  For each character the up and down keys allow the user
;       to increment or decrement the field.  Select goes to the next
;       character while menu backs up one character.  For each chracter
;       we have a table telling the position on the screen and the max
;       value (all min values are 0).  Special handling is done for the
;       N/S and E/W characters.

spstr:  lda     #22             ; message #22 (postion string)
        jsr     wrmess          ; write it
        clr     msg+18          ; the on/off flag is ALSO the end-of-string!!
        clr     keycnt          ; clear key counter
        lda     #$0e            ; cursor on command
        jsr     wrcmd           ; turn the cursor on
        clr     cntr            ; our field pointer
spwfn:  lda     curkey          ; get current key
        cmp     #knone          ; wait for no key
        bne     spwfn           ; loop till none

spstlp: lda     #$80            ; LCD postion top left
        jsr     wrcmd           ; Send command
        ldhx    #msg            ; point to our position string
        jsr     wrdats          ; go write the current string.
        ldx     cntr            ; our field pointer
        clrh                    ; make sure H is 0
        lda     spcol,x         ; get the place to put the cursor
        jsr     wrcmd           ; place it

spscan: lda     curkey          ; get key pressed (if any)
        cmp     #kwait          ; see if wait code
        beq     spscan          ; no new data yet, wait
        cmp     #ksel           ; see if select
        beq     lmenup          ; if so, off to menus
        cmp     #knone          ; if no key, reset rapid count
        beq     sprst           ; go do it
        cmp     #kup            ; see if up key
        beq     spupdn          ; go process
        cmp     #kdown          ; see if down key
        beq     spupdn          ; go do it if so

sprst:  clr     keycnt          ; clear key pressed count
        bra     spscan          ; go wait for a key

lmenup: jmp     llmenp          ; long jump

spupdn: lda     #1              ; default inc/dec by 1
        sta     temp            ; remember it
        lda     keycnt          ; get key counter
        beq     spdoit          ; if 0 value, go inc/dec by 1
        cmp     #11             ; see if 11 samples
        blo     spkwt           ; if 1-11, no action
        dec     keycnt          ; don't want keycnt to overflow
        lda     #5              ; inc/dec by 5 when held down
        sta     temp            ; remember it

spdoit: lda     cntr            ; get pointer
        cmp     #6              ; see if N/S field
        beq     spns            ; if so, go change it (up/dn irrelevant)
        cmp     #14             ; see if E/W field
        beq     spew            ; if so, go change it (up/dn irrelevant)

        lda     curkey          ; get keycode back again
        cmp     #kdown          ; see if down (vs up)
        beq     spdown          ; if so, go decrement time

spup:   ldx     cntr            ; get our counter
        clrh                    ; make sure H is 0
        ldx     spofs,x         ; point to real variable
        inc     msg,x           ; get the field to increment
        lda     msg,x           ; get incremented value
        ldx     cntr            ; get pointer back
        cmp     spmax,x         ; compare to max value
        bls     spntim          ; if within limits, all done
        lda     #'0'            ; start back over at '0'
        ldx     spofs,x         ; point to real variable
        sta     msg,x           ; otherwise clear to 0
        bra     spntim          ; all done

spdown: ldx     cntr            ; get our pointer
        clrh                    ; make sure H is 0
        ldx     spofs,x         ; point to real variable
        dec     msg,x           ; get the field to increment
        lda     msg,x           ; get decremented value
        cmp     #'0'            ; compare to min
        bhs     spntim          ; if higher or same, done!

        ldx     cntr            ; get pointer back
        lda     spmax,x         ; otherwise roll, get max value
        ldx     spofs,x         ; point to real variable
        sta     msg,x           ; save as new value
        bra     spntim          ; all done

spns:   lda     msg+8           ; get current N/S character
        cmp     #'N'            ; see if north
        beq     spnss           ; if so, change to south
        lda     #'N'            ; otherwise is N
        bra     spnsw           ; write it
spnss:  lda     #'S'            ; South flag
spnsw:  sta     msg+8           ; write it
        bra     spntim          ; all done

spew:   lda     msg+17          ; get current E/W character
        cmp     #'E'            ; see if East
        beq     spews           ; if so, change to West
        lda     #'E'            ; otherwise is East
        bra     speww           ; write it
spews:  lda     #'W'            ; West flag
speww:  sta     msg+17          ; write it
        bra     spntim          ; all done

spntim: inc     keycnt          ; increment our key counter
        lda     #kwait          ; wait for next scan flag
        sta     curkey          ; store it
lsplp:  jmp     spstlp          ; go display new time

spkwt:  inc     keycnt          ; increment key count
        lda     #kwait          ; wait for next scan flag
        sta     curkey          ; store it
        jmp     spscan          ; wait for next scan

llmenp: lda     curkey          ; get current keycode
        cmp     #knone          ; see if no key
        bne     llmenp          ; wait for no key
        inc     cntr            ; if select pressed, increment counter
        lda     cntr            ; get incremented value
        cmp     #15             ; see if done
        bne     lsplp           ; if not, loop back, else done
        lda     #$0c            ; cursor off command
        jsr     wrcmd           ; turn the cursor off

; -----------------------------------------------------------------------------
;       Done with the position string.  If MX-614 support is enabled, ask for
;       the stations callsign (MYCALL), and up to two repeater callsigns.
;       Callsign is six characters for call and one byte for SSID.
;       Callsign characters allowed are ASCII 0-9, A-Z. Six chars pad to spaces.
;       SSIDs are stored in binary format, 0x00 to 0x0f.
;       Station callsign is at msg+19 with SSID at msg+25
;       Repeater 1 callsign is at msg+26 with SSID at msg+32
;       Repeater 2 callsign is at msg+33 with SSID at msg+39
; -----------------------------------------------------------------------------
        lda     fumode          ; get the UART mode
        cmp     #04             ; see if Modem mode
        beq     spdomx          ; if so, set modem parameters
        jmp     spwrite         ; if not, skip all this stuff

spdomx: lda     msg+19          ; get first byte of callsign
        cmp     #$ff            ; see if "virgin" memory.
        beq     spsini          ; if so, go initialize to spaces
        cmp     #$00            ; other type of virgin memory
        beq     spsini          ; go initialize
        bra     spsscr          ; otherwise normal processing

;       If nothing has been initialized yet, set callsigns to spaces, and
;       SSIDs to 0.  Txdelay and Txwait get fixed later.
spsini: ldhx    #msg+19         ; the starting address
        lda     #' '            ; a space
spsccp: sta     ,x              ; clear a byte to space
        incx                    ; increment X (no overflow)
        cpx     #msg+64         ; see if end
        bne     spsccp          ; loop until end

        clra                    ;0
        sta     msg+25          ; Mycall SSID
        sta     msg+32          ; Repeater 1 SSID
        sta     msg+39          ; Repeater 2 SSID

spsscr: lda     #14             ; Big message #14, set callsign
        jsr     wr4mes          ; write it

        mov     msg+25,crc8     ; get the SSID           MYCALL
        jsr     spsssi          ; display it
        lda     #6              ; 6 characters
        ldhx    #msg+19         ; where to put them
        jsr     getext          ; get the text
        mov     msg+25,crc8     ; get the SSID again
        jsr     spssid          ; update it
        mov     crc8,msg+25     ; save the updated SSID

        ldhx    #rpt1mg         ; repeater message       REPEATER 1
        jsr     wrstr           ; write it
        mov     msg+32,crc8     ; get the SSID
        jsr     spsssi          ; display it
        lda     #6              ; 6 characters
        ldhx    #msg+26         ; where to put them
        jsr     getext          ; get the text
        mov     msg+32,crc8     ; get the SSID again
        jsr     spssid          ; update it
        mov     crc8,msg+32     ; save the updated SSID

        ldhx    #rpt2mg         ; Repeater message       REPEATER 2
        jsr     wrstr           ; write it
        mov     msg+39,crc8     ; get the SSID
        jsr     spsssi          ; display it
        lda     #6              ; 6 characters
        ldhx    #msg+33         ; where to put them
        jsr     getext          ; get the text
        lda     msg+33          ; get the first character
        cmp     #' '            ; see if a space (no callsign)
        beq     spsdly          ; if no callsign, skip SSID entry
        mov     msg+39,crc8     ; get the SSID again
        jsr     spssid          ; update it
        mov     crc8,msg+39     ; save the updated SSID

        page
;       Finally, time to set up some programmable delays:
;       FTXCLRx100ms is the time the channel should be clear before asserting
;       PTT to start a transmission.
;       FTXDLYx100ms is the time from PTT assertion to sending the packet.

spsdly: lda     #$0c            ; cursor off command
        jsr     wrcmd           ; turn the cursor off

        jsr     wnokey          ; wait for no key pressed
        lda     #52             ; message #52, Silence to PTT delay
        jsr     wrmess          ; write it
        lda     msg+40          ; get the old delay value
        jsr     setdly          ; allow user to change it
        sta     msg+40          ; save result

        jsr     wnokey          ; wait for no key pressed
        lda     #53             ; message #53, PTT to Packet delay
        jsr     wrmess          ; write it
        lda     msg+41          ; get the old delay value
        jsr     setdly          ; allow user to change it
        sta     msg+41          ; save result

        page
;       Now that the data set is complete, time to write it to Flash.
;       To keep the string write routines happy, the first byte of the
;       msg buffer is the LCD address, so the string we want to write
;       to flash starts at (msg+1).  It is a fixed 18-character string
;       plus the one-byte flag that is $00 to send the message or $ff
;       to not send it.  Flash is programmed 8 bytes at a time, so after
;       the minimum-size 128-byte erase, we program three sets of eight
;       byte buffers.
spwrite:sei                     ; interrupts off while messing with flash
        ldhx    #fpage4         ; bank 4 is for the identifer string
        jsr     appera          ; go erase it (all $ff's)
        cli                     ; allow an interrupt if it happened along
        nop                     ; waste a few clocks

        sei                     ; no more interrupts again
        ldhx    #fpage4         ; bank 4 is for the identifer string
        jsr     wrf64           ; go write all the data
        cli                     ; allow an interrupt if it happened along

        jmp     menus           ; long jump to menus

spscpy: lda     0,x             ; get first byte
        sta     mndray          ; write it to the program buffer
        lda     1,x             ; get next byte
        sta     mndray+1        ; write it
        lda     2,x             ; get next byte
        sta     mndray+2        ; write it
        lda     3,x             ; get next byte
        sta     mndray+3        ; write it
        lda     4,x             ; get next byte
        sta     mndray+4        ; write it
        lda     5,x             ; get next byte
        sta     mndray+5        ; write it
        lda     6,x             ; get next byte
        sta     mndray+6        ; write it
        lda     7,x             ; get next byte
        sta     mndray+7        ; write it
        rts                     ; done

        page
; -----------------------------------------------------------------------------
; Routine:      rdf64
; Called By:    most cal routines
; Calls:        None
; Gazintas:     (hx) Flash address to read
; Gazoutas:     (msg) 64 bytes from flash starting at (hx)
; Function:     This routine reads 64 bytes from the flash location at (hx)
;               and puts it into the 64-byte area starting at (msg).  Since
;               this routine whacks a lot of memory, it uses the special
;               space reserved during calibration procedures.
; -----------------------------------------------------------------------------
rdf64:  sthx    divdnd          ; save source
        ldhx    #msg            ; get destination address
        sthx    quotnt          ; save destination address

rdflp:  ldhx    divdnd          ; get source address
        lda     ,x              ; get source byte
        aix     #1              ; increment source address
        sthx    divdnd          ; save source address
        ldhx    quotnt          ; get destination address
        sta     ,x              ; save destination data
        aix     #1              ; increment destination address
        sthx    quotnt          ; save destination address

        cpx     #msg+64         ; check destination address LSB (MSB==0)
        bne     rdflp           ; loop till done
        rts                     ; done

; -----------------------------------------------------------------------------
; Routine:      wrf64
; Called By:    most cal routines
; Calls:
; Gazintas:     (hx) Flash address to write to
;               (flash) 64 bytes of data starting at (msg)
; Gazoutas:     None.
; Function:     This routine takes 64 bytes of data at (msg) and writes it to
;               flash starting at address (hx).  (hx) is assumed to be at the
;               start of a flash page.  Since this routine whacks a lot of
;               memory, it uses the special space reserved during calibration
;               procedures.
; -----------------------------------------------------------------------------
wrf64:  sthx    divdnd          ; save source
        ldhx    #msg            ; get destination address
        sthx    quotnt          ; save destination address

        sei                     ; interrupts off

wrflp:  ldhx    quotnt          ; get source address
        jsr     sscpmn          ; copy to mndray
        ldhx    divdnd          ; get destination address
        jsr     apppgm          ; go program it

        ldhx    divdnd          ; get destination address again
        aix     #8              ; increment source address
        sthx    divdnd          ; save destination address

        ldhx    quotnt          ; get destination address
        aix     #8              ; increment destination address
        sthx    quotnt          ; save destination address

        cpx     #msg+64         ; check destination address LSB (MSB==0)
        bne     wrflp           ; loop till done
        cli                     ; interrupts back on
        rts                     ; done

; Routine to copy 8 byte sensor address in H:X to mndray for programming

sscpmn: lda     ,x              ; get byte 1
        sta     mndray          ; store byte #1
        lda     1,x             ; get byte #2
        sta     mndray+1        ; store byte #2
        lda     2,x             ; get byte #2
        sta     mndray+2        ; store byte #2
        lda     3,x             ; get byte #2
        sta     mndray+3        ; store byte #2
        lda     4,x             ; get byte #2
        sta     mndray+4        ; store byte #2
        lda     5,x             ; get byte #2
        sta     mndray+5        ; store byte #2
        lda     6,x             ; get byte #2
        sta     mndray+6        ; store byte #2
        lda     7,x             ; get byte #2
        sta     mndray+7        ; store byte #2
        rts                     ; done

        page
; -----------------------------------------------------------------------------
; Routine:      getext
; Called By:    sposn
; Calls:        TBD
; Gazintas:     A = # of characters to get (1-35)
;               HX = address to put characters
; Gazoutas:     (hx) updated and null terminated.
; Function:     This routine gets user input for a line of data.  The data is
;               dislayed on the screen starting at the upper left corner.  If
;               the text is greater than 20 chracters, it overflows to the
;               next line.  Acceptable characters are 0-9,A-Z.
;               The string is padded out to spaces after the last chracter.
; -----------------------------------------------------------------------------
getext: sta     gtmxch          ; save max character count.
        sthx    macnv           ; save starting address
        clra                    ; 0..
        sta     gtcch           ; start at chacter #0

        lda     #$0e            ; cursor on command
        jsr     wrcmd           ; turn the cursor on

        jsr     wnokey          ; wait for no key pressed

        clr     keycnt          ; initialize key counter

;       Start by displaying the current lines contents.

gettlp: lda     #$80            ; top left corner
        jsr     wrcmd           ; send as a command

        ldhx    macnv           ; starting address
        lda     #6              ; six characters
        jsr     wrnstr          ; Write the line

;       Now put the cursor on the character to be modified

        lda     gtcch           ; get the offset
        add     #$80            ; make a cursor offset command
        cmp     #$94            ; see if > 1 line
        blo     get1l           ; if not, no mods
        add     #$40-$14        ; Line 2 offset
get1l:  jsr     wrcmd           ; tell LCD so

;       now figure out the index of the current character in txttab so we know
;       what to increment from.  If we get an illegal byte, default to "A".

        lda     macnv+1         ; get message base
        add     gtcch           ; add offset (no overflow possible)
        clrh                    ; MSB=0
        tax                     ; make an index
        lda     ,x              ; get the character

        ldhx    #txttab         ; start of the table
gettip: cbeq    x+,gettkp       ; see if match
        cphx    #txttab+36      ; see if end
        bne     gettip          ; if not, next iteration
        ldx     #$ff            ; no match flag
        bra     gett00          ; and skip reverse-lookup

;       We get here with X containg the offset of the current character in
;       the txttab array.  First wait for a keypress.  This is a special
;       keypress routine that handles repeats to allow for scrolling.

gettkp: txa                     ; get index in A
        sub     #txttab%256+1   ; subtract LSB (overflow not important)
        tax                     ; put back in X
gett00: pshx                    ; save index
        jsr     getrky          ; get the keycode
        pulx                    ; get index back

        cmp     #kup            ; see if up key
        beq     gettku          ; go process
        cmp     #kdown          ; see if down key
        beq     gettkd          ; go do it if so
        cmp     #ksel           ; see if select
        beq     gettks          ; if so, next character
        cmp     #kmenu          ; see if menu
        beq     gettkm          ; if so, next character
        bra     gettkp          ; if none, wait some more

gettku: incx                    ; increment chracter index
        cpx     #36             ; see if wrap
        blo     gettpc          ; if not, done
        clrx                    ; otherwise wrap (takes care of no-match flag too)
        bra     gettpc          ; and done

gettkd: decx                    ; decrement character index
        bpl     gettpc          ; if no underflow, exit
        ldx     #35             ; otherwise wrap
        bra     gettpc          ; save it

gettks: cpx     #$ff            ; see if a character was set
        beq     gettlp          ; if not, don't increment
        lda     gtcch           ; get index
        inca                    ; increment it
        sta     gtcch           ; save it
        cmp     gtmxch          ; see if max'd out
        bhs     gettkm          ; if so, treat as Menu (end)
        bra     gettlp          ; and on to next chacter

gettpc: clrh                    ; MSB=0
        lda     txttab,x        ; get character
        psha                    ; save it
        lda     gtcch           ; get offset
        add     macnv+1         ; add base (no overflow)
        tax                     ; make index
        clrh                    ; MSB=0
        pula                    ; get character back
        sta     ,x              ; save new character
        bra     gettlp          ; and on to next keypress

gettkm: lda     gtcch           ; get index into A
        cmp     gtmxch          ; see if at very end
        beq     gettkx          ; if so, done!

        add     macnv+1         ; add to message base (no overflow possible)
        tax                     ; index
        clrh                    ; a complete index
        lda     #' '            ; a space
        sta     ,x              ; save it
        lda     gtcch           ; get character
        add     #1              ; increment it
        sta     gtcch           ; on to next character position
        bra     gettkm          ; loop until filled with spaces

gettkx: rts                     ; done!

        page
; -----------------------------------------------------------------------------
; Routine:      spssid
; Called By:    sposn
; Calls:        TBD
; Gazintas:     crc8    Recycled byte with the SSID
; Gazoutas:     crc8    The updated SSID
; Function:     This is the routine to get the SSID.  The screen just asked for
;               the callsign, so now this replaced the word callsign with SSID
;               and lets the user select up and down to cycle through nuthin,
;               or 1-15.
; -----------------------------------------------------------------------------
spssid: ldhx    #sidmsg         ; SSID message
        jsr     wrstr           ; write it

        jsr     wnokey          ; wait for no key pressed

        lda     crc8            ; Get the (potential) SSID
        cmp     #$ff            ; see if FF (none specified yet)
        bne     spss00          ; if not, someone's fiddled with it before
        clr     crc8            ; otherwise start with 0

spss00: and     #$0f            ; LSBs only for SSID
        clr     keycnt          ; initialize key counter


spsidl: jsr     spsssi          ; show the SSID

spsidk: jsr     getrky          ; get the keycode

        cmp     #kup            ; see if up key
        beq     getsku          ; go process
        cmp     #kdown          ; see if down key
        beq     getskd          ; go do it if so
        cmp     #ksel           ; see if select
        beq     getsks          ; if so, next character
        cmp     #kmenu          ; see if menu
        beq     getsks          ; if so, next character
        bra     spsidk          ; if none, wait some more

getsku: lda     crc8            ; get SSID
        inca                    ; add one
        and     #$0f            ; wrap if needed
        sta     crc8
        bra     spsidl          ; update screen and wait for more

getskd: lda     crc8            ; get SSID
        deca                    ; add one
        and     #$0f            ; wrap if needed
        sta     crc8
        bra     spsidl          ; update screen and wait for more

getsks: lda     crc8            ; get SSID
        ora     #$30            ; add R bits (to be shifted left one)
        sta     crc8
        rts                     ; all done, return

        page
; -----------------------------------------------------------------------------
; Routine:      spsssi
; Called By:    sposn
; Calls:        TBD
; Gazintas:     crc8    Recycled byte that has the SSID
; Gazoutas:     None (the screen)
; Function:     This is the routine to display the SSID.  It takes the binary
;               value and writes nuthin (0x00) or -1 to -15 (0x01-0x0f) to show
;               the currently selected SSID.
; -----------------------------------------------------------------------------
spsssi: lda     #$87            ; the place to put the SSID
        jsr     wrcmd           ; set the cursor

        lda     crc8            ; get the SSID
        and     #$0f            ; make it 0-15
        lsla                    ; times two
        tax                     ; an index
        clrh                    ; MSB 0...
        lda     sidtab,x        ; get byte 1

        pshx                    ; save X
        pshh                    ; save H
        jsr     wrdat           ; send 1st byte
        pulh                    ; get H back
        pulx                    ; get X back
        aix     #1              ; point to byte #2
        lda     sidtab,x        ; get it
        jsr     wrdat           ; send it

        lda     #$87            ; put the cursor back
        jsr     wrcmd           ; set the cursor
        rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine:      getrky
; Called By:    getext
; Calls:        TBD
; Gazintas:     keycnt - repeat timer (zero before first call)
;               curkey - current keypress (set by timer sampler)
; Gazoutas:     A = key pressed with repeats on Up and Down.
; Function:     This routine gets called to run a keyboard scan that allows
;               repeats if the up and down keys are pressed and held down.
;               Menu and Select are treated normally.
;               This routine is called like any other normal keyboard scan
;               routine exept that it remembers if a key is still being pressed
;               from before and will start to return with a value quicker if
;               the up/down key has been being held down.
;               NOTE THAT keycnt MUST BE INITIALIZED TO ZERO BEFORE THIS GETS
;               CALLED FOR THE FIRST TIME SO THAT REPEATS DON'T HAPPEN RIGHT
;               AWAY!
; -----------------------------------------------------------------------------
getrky: lda     curkey          ; get key pressed (if any)

        cmp     #kwait          ; see if wait code
        beq     getrky          ; no new data yet, wait

        cmp     #knone          ; if no key, reset rapid count
        beq     getrst          ; go do it

        cmp     #ksel           ; see if select
        beq     getotk          ; if so, one-time key
        cmp     #kmenu          ; see if menu
        beq     getotk          ; if so, one-time key

        cmp     #kup            ; see if up key
        beq     getrud          ; go process repeat key
        cmp     #kdown          ; see if down key
        beq     getrud          ; go process repeat key

        bra     getrky          ; if none, recycle

getrst: clr     keycnt          ; clear key pressed count
;                               ; so next key is acted upon
        bra     getrky          ; go wait for a key

getotk: lda     keycnt          ; get key count
        beq     getkx           ; process same as first up/down key
        mov     #kwait,curkey   ; if 1+ samples, just wait
        bra     getrky          ; and wait for next sample

getrud: lda     keycnt          ; get key counter
        beq     getkx           ; if 0 value, go inc/dec by 1

        cmp     #11             ; see if 11+ samples
        bhs     getkx1          ; if so, respond with valid key

        mov     #kwait,curkey   ; if 1-11 samples, just wait
        inc     keycnt          ; increment sample count
        bra     getrky          ; and wait for next sample

getkx:  inc     keycnt          ; increment key count
getkx1: lda     curkey          ; get the key value back for up/down
        mov     #kwait,curkey   ; done until next scan loop
        rts                     ; return with keycode

        page
; -----------------------------------------------------------------------------
; Routine:      setdly
; Called By:    ssens
; Calls:        TBD
; Gazintas:     A = Delay value digit (1-9) in binary
; Gazoutas:     A = Delay value digit (1-9) after user adjusted it
; Function:     This routine gets called to adjust the two transmitter delay
;               values.  In case memory is un-initialized, A is reset to 5 if
;               it is out of the range 1-9.  After that, the user can press
;               up and down to adjust it within that range.  Pressing select
;               exits with the last value set.  This routine assumes all the
;               regular text is in place and the digit being adjusted is in the
;               first column of line 2 in the display.
; -----------------------------------------------------------------------------
setdly: clr     keycnt          ; re-initialize repeat counter
        tsta                    ; check incoming value
        beq     setdnw          ; if 0, it needs initializing
        cmp     #9              ; check against max value
        bls     setdlp          ; if <=9, all OK

setdnw: lda     #5              ; default value if out of range

setdlp: psha                    ; save delay value
        add     #'0'            ; convert to ASCII
        psha                    ; save this too
        lda     #$c0            ; line 2 column 1...
        jsr     wrcmd           ; set it
        pula                    ; get ASCII digit back
        jsr     wrdat           ; write it

setdbk: jsr     getrky          ; get a key (with repeats)
        cmp     #kup            ; see if up key
        beq     setdku          ; go process
        cmp     #kdown          ; see if down key
        beq     setdkd          ; go do it if so
        cmp     #ksel           ; see if select
        beq     setdks          ; if so, next character
        cmp     #kmenu          ; see if menu
        beq     setdks          ; if so, next character
        bra     setdbk          ; if none of the above, go try again.

setdku: pula                    ; get delay value
        cmp     #9              ; see if 9
        bhs     setdlp          ; if 9, at max, don't do nuthin.
        inca                    ; otherwise increment
        bra     setdlp          ; and wait for next key.

setdkd: pula                    ; get delay value
        cmp     #1              ; see if 1
        bls     setdlp          ; if 1, at min, don't do nuthin.
        deca                    ; otherwise decrement
        bra     setdlp          ; and wait for next key.

setdks: pula                    ; get delay value
        rts                     ; done!


