        page
******************************************************
** Copyright 2003 CMI Productions.                  **
** Lots of rights reserved.                         **
** Free for personal use as long as this copyright  **
** message is maintained in the source and the LCD  **
** copyright message is also preserved.  Commercial **
** use prohibited without CMIs written concent.     **
******************************************************
; -----------------------------------------------------------------------------
; File:         tempr.asm
; Function:     Temperature reading, converting, and printing routines.
; Routines:     caltmp: Do temperatures ID search and program.
;               initmp: Initialize temperature stats
;               newtmp: 5-minute re-initialization
;               iqtmp1: Start temperature conversion
;               iqtmpw: do nothing while A/D runs
;               iqtmp2: Get temperature data (and accumulate stats)
;               pitemp: convert current temperature to ASCII
;               pmtemp: convert current temperature to ASCII (mainline)
;               pntemp: convert current temperature to ASCII (metric)
;               pitavg: compute average temperature & convert to ASCII
;               avgtmp: Compute averaged temperature
; -----------------------------------------------------------------------------

; -----------------------------------------------------------------------------
; Routine:      caltmp
; Called By:    ssens
; Calls:        isrch (find ID)
;               wrmess (write LCD message to LCD)
;               waits (wait)
;               cnvid (convert ID to ASCII)
;               wrstr (write LCD null-terminated string)
;               wrcmd (write LCD command byte)
;               wrdat (write LCD data byte)
;               wfnkey (wait for next key press)
;               rdf64 (read 64-byte Flash block
;               wrf64 (write 64-byte flash block)
;               appera (erase flash block)
; Gazintas:     None
; Gazoutas:     Flash location fitmp is programmed
; Function:     This is the temperature sensor finder program.  For this code
;               I explain each code block as I go.
; -----------------------------------------------------------------------------
caltmp:
;       Go out and get the temperature sensor ID.  Here I check to see if
;       there are two sensors out there.  This is possible if the user leaves
;       the old bad DS1820 in the outdoor unit, then adds the DS18S20 as
;       an external unit.  If this happens, I will present both IDs and let
;       the user pick which one is the correct one.

        clr     flg12           ; start off one way...
        lda     #$10            ; temperature sensor family ID
        jsr     isrch           ; go find an ID, put into mndray
        tst     mndray          ; make sure we got something
        bne     caltok          ; if so, on to see what it was

caltns: lda     #46             ; No Sensor message
        jsr     wrmess          ; write it
        lda     #40             ; 4 seconds
        jsr     waits           ; wait it
        rts                     ; and exit

caltok: ldhx    mndray          ; get ID bytes 1-2
        sthx    msg+20          ; save 'em
        ldhx    mndray+2        ; get ID bytes 3-4
        sthx    msg+22          ; save 'em
        ldhx    mndray+4        ; get ID bytes 5-6
        sthx    msg+24          ; save 'em
        ldhx    mndray+6        ; get ID bytes 7-8
        sthx    msg+26          ; save 'em

        inc     flg12           ; see if another counter ID
        lda     #$10            ; temperature sensor family ID
        jsr     isrch           ; go find an ID, put into mndray
        tst     mndray          ; make sure we got something
        beq     caltns          ; if not, weird error, abort

        ldhx    mndray          ; get 2nd reading bytes 1-2
        cphx    msg+20          ; see if same as 1st reading
        bne     sst2            ; if different, two sensors!
        ldhx    mndray+2        ; get 2nd reading bytes 3-4
        cphx    msg+22          ; see if same as 1st reading
        bne     sst2            ; if different, two sensors!
        ldhx    mndray+4        ; get 2nd reading bytes 5-6
        cphx    msg+24          ; see if same as 1st reading
        bne     sst2            ; if different, two sensors!
        ldhx    mndray+6        ; get 2nd reading bytes 7-8
        cphx    msg+26          ; see if same as 1st reading
        bne     sst2            ; if different, two sensors!
        bra     ssts            ; if all same, go program ID.

;       We get here if there are two sensors.  Simply plunk them on the screen
;       and let the user pick.
sst2:   lda     #31             ; message #31 (Pick your sensor)
        jsr     wrmess          ; write it

        ldhx    #msg+20         ; point to first ID to convert to ASCII
        jsr     cnvid           ; go convert to ASCII in (msg+1)
        mov     #$94,msg        ; line 3
        ldhx    #msg            ; point to the whole message
        jsr     wrstr           ; go write it

        ldhx    #mndray         ; point to second ID to convert to ASCII
        jsr     cnvid           ; go convert to ASCII in (msg+1)
        mov     #$d4,msg        ; line 3
        ldhx    #msg            ; point to the whole message
        jsr     wrstr           ; go write it

        clr     cntr            ; our flag as to pick the ID

ssq4lp: tst     cntr            ; test the A/B flag
        bne     ssq4b           ; if not 0, B selection
ssq4a:  lda     #$a5            ; just after first ID text
        jsr     wrcmd           ; say there
        lda     #'*'            ; the marker
        jsr     wrdat           ; mark it
        lda     #$e5            ; just after second ID text
        jsr     wrcmd           ; say there
        lda     #' '            ; the NOT marker
        jsr     wrdat           ; mark it
        bra     ssq4            ; on to user feedback

ssq4b:  lda     #$a5            ; just after first ID text
        jsr     wrcmd           ; say there
        lda     #' '            ; the NOT marker
        jsr     wrdat           ; mark it
        lda     #$e5            ; just after second ID text
        jsr     wrcmd           ; say there
        lda     #'*'            ; the marker
        jsr     wrdat           ; mark it

ssq4:   jsr     wfnkey          ; wait for next keypress
        cmp     #ksel           ; see if select button (continue)
        beq     ssq4c           ; if so, go program selected ID
        cmp     #kup            ; see if up
        beq     ssq4ud          ; go process up/down (same)
        cmp     #kdown          ; see if up
        beq     ssq4ud          ; go process up/down (same)
        bra     ssq4            ; if neither, ignore.

ssq4ud: com     cntr            ; complement our A/B flag
        bra     ssq4lp          ; and go show that

ssq4c:  tst     cntr            ; see which one user selected
        bne     ssts            ; if B, requested ID is in (mndray), so done

        ldhx    msg+20          ; otherwise copy (msg+20) to (mndray)
        sthx    mndray          ; store bytes 0,1
        ldhx    msg+22          ; get bytes 2,3
        sthx    mndray+2        ; store 'em
        ldhx    msg+24          ; get bytes 2,3
        sthx    mndray+4        ; store 'em
        ldhx    msg+26          ; get bytes 2,3
        sthx    mndray+6        ; store 'em

;       We get here if there was only one temperature sensor found or the
;       conflict was resolved by the user.  The address in mndray is the one
;       to be programmed.
;       To update the flash block that has other addresses we don't want to
;       mess with we copy the entire block to (msg), modify the temperature
;       address, then write the entire block back.
ssts:   ldhx    #fpage0         ; the page to read
        jsr     rdf64           ; read it

        ldhx    mndray          ; get bytes 0,1
        sthx    msg             ; store them
        ldhx    mndray+2        ; get bytes 0,1
        sthx    msg+2           ; store them
        ldhx    mndray+4        ; get bytes 0,1
        sthx    msg+4           ; store them
        ldhx    mndray+6        ; get bytes 0,1
        sthx    msg+6           ; store them

        sei                     ; interrupts off
        ldhx    #fpage0         ; point to whole page again
        jsr     appera          ; erase it
        ldhx    #fpage0         ; point to whole page again
        jsr     wrf64           ; program altered values

        cli                     ; interrupts back on
        rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine:      initmp,newtmp
; Called By:    timrtn
; Calls:        None
; Gazintas:     None
; Gazoutas:     cumtmp
;               tsamp
; Function:     Same code for first-time use and 5-minute re-use.
;               This is the routine to initialize the variables needed for
;               the averaged temperature readings.  Since we simply maintain a
;               sum of the temperature readings and sample count, all we do is
;               zero those out and we are all set.
; -----------------------------------------------------------------------------
initmp:
newtmp: clr     cumtmp          ; clear totaled MSB
        clr     cumtmp+1        ; clear totaled LSB
        clr     tsamp           ; clear temperatuare sample counter
        rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine:      iqtmp1
; Called By:    timrtn
; Calls:        sndadr (send a 1-wire address)
;               i_writ (send a 1-wire byte)
; Gazintas:     None
; Gazoutas:     errtmp (error flags/counter)
; Function:     Starts the temperature conversion process by sending the
;               convert command to the temperature sensor and then asserting
;               the 5V strongly.  We also clear out any old error flags as
;               we are starting anew.
; -----------------------------------------------------------------------------
iqtmp1: bclr    7,errtmp        ; clear any errors to start fresh
        ldhx    #fitemp         ; point to the temperature ibutton address
        jsr     sndadr          ; go send it
        lda     #$44            ; convert temperature command
        jsr     i_writ          ; send it

        mov     #$86,ddrb       ; want high power mode
        rts                     ; all done



; -----------------------------------------------------------------------------
; Routine:      iqtmpw
; Called By:    timrtn
; Calls:        None
; Gazintas:     None
; Gazoutas:     None
; Function:     None.  This is a dummy routine that gets executed for seven
;               timer ticks while we are waiting for the A/D conversion to
;               complete.
; -----------------------------------------------------------------------------
iqtmpw: rts                     ; not much, eh?


        page
; -----------------------------------------------------------------------------
; Routine:      iqtmp2
; Called By:    timrtn
; Calls:        sndadr (send a 1-wire address)
;               i_read (read a 1-wire byte)
;               docrc8 (do 8-bit CRC on data)
; Gazintas:     None
; Gazoutas:     errtmp (temperature error flag/counter)
;               rdtemp (temperature sample)
;               tsamp (temperature sample counter)
; Internals:    crc8 (8-bit CRC tallier)
; Function:     Now that the command to convert and the 750ms delay are
;               done, it is time to get the data.  First quit actively driving
;               the 1-wire bus high so we can start communicating again.  After
;               that, we go read the block of data that contains the
;               temperature reading and a CRC result.  The result is in 2's
;               complement 7.1 format which is a pain, so I convert it to
;               offset-110 (55 degrees C) which is a lot easier to deal with.
;
;               A reading of $aa is a new flag on the DS18S20 that means some
;               kind of error occurred.  I now check for this.
; -----------------------------------------------------------------------------
;       iqtmp2: With the conversion complete, This routine goes and gets the
;       data.  It gets checked for validity, and then converted to an
;       offset-110 for later processing

iqtmp2: mov     #$80,ddrb       ; high power mode off

        ldhx    #fitemp         ; point to the temperature iButton address
        jsr     sndadr          ; send it
        lda     #$be            ; read scratchpad command
        jsr     i_writ          ; send it

        clr     crc8            ; clear the CRC checker
        jsr     i_read          ; read a byte (the temperature LSB)
        sta     rdtemp          ; save raw value for a bit
        jsr     docrc8          ; do the CRC
        jsr     i_read          ; read junker byte for CRC verification
        jsr     docrc8          ; do the CRC
        jsr     i_read          ; read junker byte for CRC verification
        jsr     docrc8          ; do the CRC
        jsr     i_read          ; read junker byte for CRC verification
        jsr     docrc8          ; do the CRC
        jsr     i_read          ; read junker byte for CRC verification
        jsr     docrc8          ; do the CRC
        jsr     i_read          ; read junker byte for CRC verification
        jsr     docrc8          ; do the CRC
        jsr     i_read          ; read junker byte for CRC verification
        jsr     docrc8          ; do the CRC
        jsr     i_read          ; read junker byte for CRC verification
        jsr     docrc8          ; do the CRC
        jsr     i_read          ; read the sensors CRC
        jsr     docrc8          ; do the CRC on the CRC (answer better be 0)
        lda     crc8            ; get the CRC
        beq     rtmpok          ; if 0, life is (probably) good

rtmpaa: inc     errtmp          ; increment the temperature error counter
        bset    7,errtmp        ; set MSB for error-now flag
        bra     iqtp2x          ; and leave without doing stats

rtmpok: lda     rdtemp          ; get real value back
        cmp     #$aa            ; see if special error code
        beq     rtmpaa          ; if so, go do error processing
        add     #110            ; convert 2's comp to offset-110
        sta     rdtemp          ; save it
        inc     tsamp           ; increment temperature sample counter
iqtp2x: rts                     ; all done!

        page
; -----------------------------------------------------------------------------
; Routine:      tmpstt
; Called By:    timrtn
; Calls:        None
; Gazintas:     errtmp (temperature error flag/counter)
;               ddtemp (current temperature reading)
; Gazoutas:     temphi (current high temperature)
;               templo (current low temperature)
;               cumtmp (cumulative temperature samples)
; Function:     This routine does the stats on the samples as they come in.
;               If the current sample is bad, the data is ignored.
;               If the current temperature is higher than the high, it copied.
;               If the current temperature is lower than the low, it copied.
;               The samples are added up so we can get an average
;               We increment the sample counter to count good samples.
; -----------------------------------------------------------------------------
tmpstt: brset   7,errdd,rtempx  ; if bad data, don't do nuthin'

        lda     temphi          ; get current high
        cmp     #$ff            ; see if un-initialized value
        bne     rdtreg          ; if not, regular processing

        lda     ddtemp          ; 1st reading, just get current temp
        sta     temphi          ; make it the high
        sta     templo          ; make it the low too
        bra     rtempc          ; all done

rdtreg: lda     ddtemp          ; get current temperature
        cmp     temphi          ; compare to current high
        bls     rdtnnh          ; if less or same, no new high
        sta     temphi          ; otherwise new high
rdtnnh: cmp     templo          ; compare to current low
        bhs     rtempc          ; if higher or same, no new low
        sta     templo          ; otherwise new low

rtempc: lda     ddtemp          ; get the current temperature again
        add     cumtmp+1        ; add to cumulative temperature LSB
        sta     cumtmp+1        ; store result
        bcc     rtempx          ; if no carry, done
        inc     cumtmp          ; otherwise increment MSB
rtempx: rts                     ; all done

        page
; -----------------------------------------------------------------------------
; Routine:      pitemp
; Called By:    tncmsg
; Calls:        bin2as
; Gazintas:     cnv2a (binary temperature in C 7.1 format, *NOT* offset 110)
; Gazoutas:     atemp (temperature in ASCII)
; Function:     This is the routine to convert the temperature we get in degrees
;               C to degrees F.  I use fixed-point arithmatic to do this.  The C
;               temperature is in 7.1 format (seven digits before decimal, one
;               after).  The 9/5 ratio is approximately 1 and 204.8/256ths, or
;               1.1100111.  When we multiply the two together we get at 16-bit
;               answer in an 8.8 format I look at the fractional part and do
;               appropriate rounding.  Negative signes are extracted and added
;               after the multiplication.  At temperatures above 123 degrees C,
;               the math will overflow, but that is outside the atmospheric
;               ranges, so I won't worry about it.  Note on math errors for the
;               purists out there:  The multiplier for C to F is 1.8046875
;               instead of 1.8000000.  At the maximum absolute value of
;               temperatre (+125 degrees C), this represents an error of 0.6
;               degrees F.  This is small enough to ignore, considering the
;               person wanting to know this ambient temperature would be dead.
;               On a 125 degree F day, that is an error of 0.24 degrees
; -----------------------------------------------------------------------------
pitemp: lda     cnv2a           ; get the C temp
        bpl     tpcpos          ; if positive, no action required
        nega                    ; negate to get positive numbers

tpcpos: ldx     #$e7            ; 9/5ths in 1.7 format (1 206/256ths)
        mul                     ; multiply the two.  answer is x:a

;       We now have the integer part in X and the fractional part in A
;       We arn't going to bother with precision less than a degree, so
;       our rounding consists of looking at the sign bit of the fractional
;       part and if it is 1 (0.5 or greater), we increment X.

        tsta                    ; set N to see if >.5
        bpl     ptenoi          ; if not, don't increment X
        incx                    ; otherwise increment it
ptenoi: txa                     ; move result to A

;       A now contains the absolute value in C times 9/5.  I now need to
;       get my sign back so I can add 32.

        tst     cnv2a           ; get the original temperature back
        bpl     ptenxx          ; if positive, no action required
        nega                    ; otherwise negate
ptenxx: add     #32             ; add 32 to result
        sta     temp            ; remember it for a bit

;       Temp now contains the temperature in degrees F.  Time to start
;       converting it to ASCII text.  The bin2as function does this on a
;       16-bit value in the format temp2:temp.  So, clear temp2 to make
;       sure the MSB is 0.

        lda     temp            ; get the F temp
        bpl     tpfpos          ; if positive, no action required
        nega                    ; make it positive now

tpfpos: sta     divdnd+1        ; store LSB in recycled divdnd byte
        clr     divdnd+0        ; MSB=0
        jsr     bin2as          ; result now in atemp

;       Now that the temperature has been converted to ASCII, if it is
;       negative, go put in the negative sign.

ptenk0: lda     temp            ; get original binary temp back
        bpl     ptex            ; if positive, all done
        lda     atemp+1         ; look at 10's digit
        cmp     #' '            ; see if a space
        beq     neg10s          ; if so, negative sign in 10's column
        lda     #'-'            ; negative sign
        sta     atemp           ; if <-9, negative in 100's column
        bra     ptex            ; all done
neg10s: lda     #'-'            ; negative sign
        sta     atemp+1         ; if >-9, negative in 10's column

ptex:   rts                     ; return to caller

        page
; -----------------------------------------------------------------------------
; Routine:      pmtemp
; Called By:    showit
; Calls:        bin2am
; Gazintas:     macnv (binary temperature in C 7.1 format, *NOT* offset 110)
; Gazoutas:     matemp (temperature in ASCII)
; Function:     This is the mainline version of pitemp (no variable conflicts)
; -----------------------------------------------------------------------------
pmtemp: lda     macnv           ; get the C temp
        bpl     tmcpos          ; if positive, no action required
        nega                    ; negate to get positive numbers

tmcpos: ldx     #$e7            ; 9/5ths in 1.7 format (1 206/256ths)
        mul                     ; multiply the two.  answer is x:a

;       We now have the integer part in X and the fractional part in A
;       We arn't going to bother with precision less than a degree, so
;       our rounding consists of looking at the sign bit of the fractional
;       part and if it is 1 (0.5 or greater), we increment X.

        tsta                    ; set N to see if >.5
        bpl     pmenoi          ; if not, don't increment X
        incx                    ; otherwise increment it
pmenoi: txa                     ; move result to A

;       A now contains the absolute value in C times 9/5.  I now need to
;       get my sign back so I can add 32.

        tst     macnv           ; get the original temperature back
        bpl     pmenxx          ; if positive, no action required
        nega                    ; otherwise negate
pmenxx: add     #32             ; add 32 to result
        sta     mtemp           ; remember it for a bit

;       Temp now contains the temperature in degrees F.  Time to start
;       converting it to ASCII text.  The bin2as function does this on a
;       16-bit value in the format temp2:temp.  So, clear temp2 to make
;       sure the MSB is 0.

        lda     mtemp           ; get the F temp
        bpl     tmfpos          ; if positive, no action required
        nega                    ; make it positive now

tmfpos: sta     mbcnv+1         ; store LSB in recycled divdnd byte
        clr     mbcnv+0         ; MSB=0
        jsr     bin2am          ; result now in atemp

;       Now that the temperature has been converted to ASCII, if it is
;       negative, go put in the negative sign.

pmenk0: lda     mtemp           ; get original binary temp back
        bpl     pmex            ; if positive, all done
        lda     matemp+1        ; look at 10's digit
        cmp     #' '            ; see if a space
        beq     neg10m          ; if so, negative sign in 10's column
        lda     #'-'            ; negative sign
        sta     matemp          ; if <-9, negative in 100's column
        bra     pmex            ; all done
neg10m: lda     #'-'            ; negative sign
        sta     matemp+1        ; if >-9, negative in 10's column

pmex:   rts                     ; return to caller

        page
; -----------------------------------------------------------------------------
; Routine:      pntemp
; Called By:    showit
; Calls:        bin2am
; Gazintas:     macnv (binary temperature in C 7.1 format, *NOT* offset 110)
; Gazoutas:     matemp (temperature in ASCII)
; Function:     This is the metric mainline version of pmtemp
; -----------------------------------------------------------------------------
pntemp: lda     macnv           ; get the C temp
        bpl     tncpos          ; if positive, no action required
        nega                    ; negate to get positive numbers

tncpos: add     #1              ; some pre-emtive rounding
        lsra                    ; temperature is actually x2, so divide
        sta     mbcnv+1         ; store LSB in recycled divdnd byte
        clr     mbcnv+0         ; MSB=0
        jsr     bin2am          ; result now in atemp

;       Now that the temperature has been converted to ASCII, if it is
;       negative, go put in the negative sign.

        lda     macnv           ; get original binary temp back
        bpl     pnex            ; if positive, all done
        lda     matemp+1        ; look at 10's digit
        cmp     #' '            ; see if a space
        beq     neg10n          ; if so, negative sign in 10's column
        lda     #'-'            ; negative sign
        sta     matemp          ; if <-9, negative in 100's column
        bra     pnex            ; all done
neg10n: lda     #'-'            ; negative sign
        sta     matemp+1        ; if >-9, negative in 10's column

pnex:   rts                     ; return to caller

        page
; -----------------------------------------------------------------------------
; Routine:      avgtmp
; Called By:    tncmsg
; Calls:        divu (16/8 unsigned divide)
;               pitemp (convert binary temperature to ASCII)
; Gazintas:     cumtmp (cumulative temperature samples)
;               tsamp (total temperature samples)
; Gazoutas:     atemp (ASCII temperature)
; Function:     This is the routine to get the average temperature for the past
;               5 minutes.  We just add up the temperatures for 60 5-second
;               samples while we are accumulating data, then divide them by
;               60 here before going through the regular conversion routines.
;               Note that the actual number of samples may be different than
;               60 due to errors.  We divide by actual samples.
; -----------------------------------------------------------------------------
avgtmp: lda     cumtmp          ; get cumulative speed MSB
        sta     divdnd          ; store for division
        lda     cumtmp+1        ; get cumulative speed LSB
        sta     divdnd+1        ; store it for division
        lda     tsamp           ; the temperature sample count
        beq     avt00           ; if no samples, problems!
        sta     divsor          ; set it up

        jsr     divu            ; do the division

        lda     quotnt+1        ; get quotient LSB
        sub     #110            ; convert to two's comp
        sta     cnv2a           ; store it
        jsr     pitemp          ; convert to ASCII
        rts                     ; all done

avt00:  jsr     inidot          ; all dots if invalid data
        rts                     ; all done

